package codemaptournament;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.border.Border;

/**
 *
 * @author Paul
 */
public class BorderFlasher implements ActionListener {
    public Color borderColor;
    public int borderThickness;
    public final JComponent component;
    public int delay;
    private Border oldBorder;
    private Timer timer;
    private List<ActionListener> completedActionListeners;

    public BorderFlasher(JComponent component, int delay, Color borderColor, int borderThickness) {
        this.borderColor = borderColor;
        this.borderThickness = borderThickness;
        this.component = component;
        this.delay = delay;
        completedActionListeners = new ArrayList<ActionListener>();
        timer = new Timer(delay, this);
        timer.setRepeats(false);
    }

    public BorderFlasher(JComponent component, int delay) {
        this(component, delay, Color.red, 5);
    }

    public BorderFlasher(JComponent component) {
        this(component, 500);
    }

    public void flash() {
        if (timer.isRunning()) return;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                flashOn();
            }
        });
    }

    private void flashOn() {
        if (component != null) {
            oldBorder = component.getBorder();
            component.setBorder(BorderFactory.createLineBorder(borderColor, borderThickness));
        }
        timer.start();
    }

    private void flashOff() {
        if (component != null) {
            component.setBorder(oldBorder);
        }
        fireCompleted();
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == timer)
            flashOff();
        else
            flashOn();
    }

    public void addCompletionActionListener(ActionListener al) {
        completedActionListeners.add(al);
    }
    private void fireCompleted() {
        for (ActionListener actionListener : completedActionListeners) {
            actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, null));
        }
    }
}
