package codemaptournament;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Hold the tournament metainfo (title, etc) and maps. Lazily fetch the maps.
 * @author Paul
 */
public final class ForumTournament {
    public final WeekInfo weekInfo;

    /** null means unloaded, empty means none found, non-empty means some missions found. */
    private List<Mission> missions;

    public ForumTournament(WeekInfo weekInfo, boolean loadImmediately) {
        this.weekInfo = weekInfo;
        this.missions = null;
        if (loadImmediately) {
            readMissions();
        }
    }

    public void readMissions() {
        try {
            missions = AutoWeeklyMissionFetcher.getMissionsForWeek(weekInfo);
        } catch (IOException ex) {
            missions = new ArrayList<Mission>();
            Logger.getLogger(ForumTournament.class.getName()).log(Level.SEVERE, "Can't read URL", ex);
        }
    }

    public boolean willNeedToFetch() {
        return missions == null;
    }

    public List<Mission> getMissions() {
        if (missions == null) readMissions();
        return missions;
    }

    @Override
    public String toString() {
        return weekInfo.toString();
    }
}
