package codemaptournament;

import java.awt.Desktop;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

/**
 * A JLabel that looks and behaves like a hyperlink
 * from http://stackoverflow.com/a/4518502/1816736 by dimo414
 * Can have special behavior and appearance for click on empty link.
 * @author Paul
 */
public class JLabelLink extends JLabel implements MouseListener {
    public String EMPTY_TEXT = "(none yet)"; /** text to display instead of empty string */
    public static final URI EMPTY_URI = URI.create("");
    private String text;
    private URI uri;
    /** listeners for click on "empty link" */
    private List<ActionListener> emptyActionListeners;

    public JLabelLink() {
        this("", EMPTY_URI);
    }
    public JLabelLink(String text, String uri) {
        this(text, URI.create(uri));
    }

    public JLabelLink(String text, URI uri) {
        super();
        this.text = text;
        this.uri = uri;
        JLabelLink.this.setText(text);
        setURI(uri);
        setToolTipText(uri.toString());
        addMouseListener(JLabelLink.this);
        emptyActionListeners = new ArrayList<ActionListener>();
    }
    public void setText(String text, boolean ul){
        this.text = text;

        if (text == null || text.equals("")) {
            text = EMPTY_TEXT;
        }
        String link = !ul ? "<u>"+text+"</u>" : text;
        super.setText("<html><span style=\"color: #000099;\">"+link+"</span></html>");
        //super.setText("<html><span>Player Standings</span></html>");
    }

    @Override
    public void setText(String text){
        setText(text,true);
    }

    public void setURI(URI uri) {
        this.uri = uri;
    }

    public URI getURI() {
        return uri;
    }
    
    public String getRawText(){
        return text;
    }

    public void addEmptyActionListener(ActionListener al) {
        emptyActionListeners.add(al);
    }

    private void open(URI uri) {
        try {
            if (uri == null || uri.equals(EMPTY_URI)) {
                for (ActionListener actionListener : emptyActionListeners) {
                    actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "empty"));
                }
            } else {
                if (Desktop.isDesktopSupported()) {
                    Desktop desktop = Desktop.getDesktop();
                    if (desktop.isSupported(Desktop.Action.BROWSE)) {
                        desktop.browse(uri);
                    }
                }
            }
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(null, "Can't open URL.\n"+ex, "Browse Failure!", JOptionPane.ERROR_MESSAGE);
        }
    }

    @Override
    public void mouseClicked(MouseEvent e) {
        open(uri);
    }
    @Override
    public void mouseEntered(MouseEvent e) {
        setText(text,false);
    }
    @Override
    public void mouseExited(MouseEvent e) {
        setText(text,true);
    }

    @Override
    public void mousePressed(MouseEvent e) {
    }

    @Override
    public void mouseReleased(MouseEvent e) {
    }
}
