package codemaptournament;

/**
 * Limit something to no more than once every millisBetween.
 * To use, call proceed() before doing your something.
 * @author Paul
 */
public class RateLimiter {
    public final long millisBetween;
    private long busyUntil;

    public RateLimiter(long millisBetween) {
        this.millisBetween = millisBetween;
        busyUntil = System.currentTimeMillis();
    }

    /**
     * Delay, if needed, so that calls to proceed() are at least
     * millisBetween milliseconds apart.
     */
    public void proceed() {
        long now = System.currentTimeMillis();
        // Any need to delay?
        if (busyUntil > now) {
            try {
                // Sleep for busyUntil-now millis so that the current time will be
                // about busyUntil. This is just approximate.
                // System.out.println("Waiting for " + (busyUntil - now) + "ms for rate limit of " + millisBetween + "ms");
                Thread.sleep(busyUntil - now);
            } catch (InterruptedException ex) {}
        }
        busyUntil = now + millisBetween;
    }
}
