package codemaptournament;

import codemaptournament.SortUtil.SortSpec;
import java.util.List;
import java.util.Map;

/**
 *
 * @author Paul
 */
public class Reports {

    /** optional header to the tournament map list */
    public static String getTournamentPreamble(MarkupRenderer renderer) {
        StringBuilder sb = new StringBuilder();
        sb.append("(name thread like: Weekly 5-Map Tournament - THEME - closing DD MMM YYYY)");
        sb.append(renderer.newline());
        sb.append("INTRO GOES HERE");
        sb.append(renderer.newline());
        sb.append(renderer.newline());
        sb.append(renderer.bold("A few useful links:"));
        sb.append(renderer.newline());
        sb.append(renderer.url("http://knucklecracker.com/wiki/doku.php?id=cw2:weekly_5_map_tournament", "All Tournaments (wiki)"));
        sb.append(renderer.newline());
        sb.append(renderer.url("http://knucklecracker.com/creeperworld2/viewscores.php", "Code Map Scoreboard"));
        sb.append(renderer.newline());
        sb.append(renderer.url("http://knucklecracker.com/forums/index.php?topic=7477.0", "Tournament Score Tool"));
        sb.append(renderer.newline());
        sb.append(renderer.newline());
        sb.append(renderer.bold("Theme:"));
        sb.append(renderer.newline());
        sb.append("THEME GOES HERE");
        sb.append(renderer.newline());
        sb.append(renderer.newline());
        return sb.toString();
    }

    /**
     * Generate the high score list in a forum-formatted table.
     * Format based on part by what UpperKEES posted in Chronom high scores.
     * @param highScores score data
     * @param scoreTimestampStr record when the scores were recorded
     * @return string suitable for pasting into a forum post
     */
    public static String makeForumHighScoreList(List<HighScoreEntry> highScores, String scoreTimestampStr, boolean boldTop3, MarkupRenderer renderer) {
        StringBuilder report = new StringBuilder();

        report.append(renderer.large(renderer.bold("Tournament Results")));
        report.append(renderer.newline());
        report.append(renderer.newline());

        final int numHighScores = highScores.size();
        final int cols = 3; // rank, player, score
        final int rows = numHighScores + 1; // header + 0 hr rows
        String[][] table = new String[rows][cols];
        int col;

        // header
        int row = 0;
        col = 0;
        table[row][col++] = renderer.bold("Rank");
        table[row][col++] = renderer.bold("Player");
        table[row][col++] = renderer.bold("Total Score");
        row++;

        // high score table
        for (int i = 0; i < numHighScores; i++) {
            HighScoreEntry entry = highScores.get(i);
            String rank = Integer.toString(i + 1) + ".";
            String player = entry.user;
            String score = Integer.toString(entry.totalScore);
            if (!entry.playedAllGames) {
                score += "*";
            }
            if (i < MarkupRenderer.VICTOR_COLORS.length) {
                // special colors for first 3 places
                String color = MarkupRenderer.VICTOR_COLORS[i];
                rank = renderer.color(rank, color);
                player = renderer.color(player, color);
                score = renderer.color(score, color);
                // Optionally bold anything with a color
                if (boldTop3) {
                    rank = renderer.bold(rank);
                    player = renderer.bold(player);
                    score = renderer.bold(score);
                }
            }
            col = 0;
            table[row][col++] = rank;
            table[row][col++] = player;
            table[row][col++] = score;
            row++;
        }
        report.append(renderer.table(table, true));

        // footer
        report.append(renderer.italic("* Did not finish all maps."));
        report.append(renderer.newline());
        report.append(renderer.newline());
        report.append("Scores recorded at ").append(scoreTimestampStr).append(" forum time.");

        return report.toString();
    }

    /**
     * Generate a list of the missions in the tournament (suitable for posting
     * to start the tournament).
     * @param missionTableModel list of mission
     * @return string suitable for pasting into a forum post
     */
    public static String makeForumTournamentList(List<Mission> missions, MarkupRenderer renderer) {
        StringBuilder report = new StringBuilder();

        final int numMissions = missions.size();
        final int cols = 3; // for map name, size, complexity
        final int rows = numMissions + 1; // 5 missions + 1 header
        String[][] table = new String[rows][cols];
        int col;

        // the header
        int row = 0;
        col = 0;
        table[row][col++] = renderer.color(renderer.bold("Map Name:"), renderer.COLOR_MAPNAME);
        table[row][col++] = renderer.color(renderer.bold("Size:"), renderer.COLOR_SIZE);
        table[row][col++] = renderer.color(renderer.bold("Complexity:"), renderer.COLOR_COMPLEXITY);
        row++;

        // a row for each mission
        for (Mission mission : missions) {
            col = 0;
            table[row][col++] = MarkupRenderer.COLOR_MAP_NAMES ? renderer.color(mission.code, renderer.COLOR_MAPNAME) : mission.code;
            table[row][col++] = renderer.color(Mission.sizeStrings[mission.size], renderer.COLOR_SIZE);
            table[row][col++] = renderer.color(Mission.complexityStrings[mission.complexity], renderer.COLOR_COMPLEXITY);
            row++;
        }

        // report is just the table no fancy stuff
        report.append(renderer.table(table, true));
        return report.toString();
    }
    /**
     * Generate a list of the missions in the tournament (suitable for posting
     * to start the tournament). This has more columns of data: Gravity, Build, and Experimentals
     * @param missionTableModel list of mission
     * @return string suitable for pasting into a forum post
     */
    public static String makeForumTournamentListExtended(List<Mission> missions, MarkupRenderer renderer) {
        StringBuilder report = new StringBuilder();

        final int numMissions = missions.size();
        final int cols = 6; // for map name, size, complexity, gravity, build dir, experimentals
        final int rows = numMissions + 1; // 5 missions + 1 header
        String[][] table = new String[rows][cols];
        int col;

        // the header
        int row = 0;
        col = 0;
        table[row][col++] = renderer.color(renderer.bold("Map Name:"), MarkupRenderer.COLOR_MAPNAME);
        table[row][col++] = renderer.color(renderer.bold("Size:"), MarkupRenderer.COLOR_SIZE);
        table[row][col++] = renderer.color(renderer.bold("Complexity:"), MarkupRenderer.COLOR_COMPLEXITY);
        table[row][col++] = renderer.color(renderer.bold("Gravity:"), MarkupRenderer.COLOR_GRAVITYDIR);
        table[row][col++] = renderer.color(renderer.bold("Build:"), MarkupRenderer.COLOR_BUILDDIR);
        table[row][col++] = renderer.color(renderer.bold("Exp:"), MarkupRenderer.COLOR_HASEXPERIMENTALS);
        row++;

        // a row for each mission
        for (Mission mission : missions) {
            col = 0;
            table[row][col++] = MarkupRenderer.COLOR_MAP_NAMES ? renderer.color(mission.code, MarkupRenderer.COLOR_MAPNAME) : mission.code;
            table[row][col++] = renderer.color(Mission.sizeStrings[mission.size], MarkupRenderer.COLOR_SIZE);
            table[row][col++] = renderer.color(Mission.complexityStrings[mission.complexity], MarkupRenderer.COLOR_COMPLEXITY);
            table[row][col++] = renderer.color("DIRECTION", MarkupRenderer.COLOR_GRAVITYDIR);
            table[row][col++] = renderer.color("DIRECTION", MarkupRenderer.COLOR_BUILDDIR);
            table[row][col++] = renderer.color("YES/NO", MarkupRenderer.COLOR_HASEXPERIMENTALS);
            row++;
        }

        // report is just the table no fancy stuff
        report.append(renderer.table(table, true));
        return report.toString();
    }

    // how the columns of the player results tables should be sorted for best 3
    // the order is relative to the table model - the JTable (view) might
    // have these columns in a different order.
    // TODO add sort by # of maps played first (maybe by hidden row in table,
    // or by HighScoreEntry and Player Results). See http://knucklecracker.com/forums/index.php?topic=7477.msg84544#msg84544.
    private final static SortSpec [] modelSortSpecs = {
        SortSpec.DONT_CARE, // col 0 - title
        SortSpec.HI_FIRST, // col 1 - map 1
        SortSpec.HI_FIRST, // col 2 - map 2
        SortSpec.HI_FIRST, // col 3 - map 3
        SortSpec.HI_FIRST, // col 4 - map 4
        SortSpec.HI_FIRST, // col 5 - map 5
        SortSpec.HI_FIRST, // col 6 - total scores
        SortSpec.LO_FIRST, // col 7 - total times
    };
    /**
     * Table of results for each map. Optionally color and bold the top 3 scores.
     */
    public static String makeForumDetailedResults(Map<String, Map<Integer,Record>> scoresByGameByUser, List<HighScoreEntry> highScores, List<Mission> missions, String scoreTimestampStr, boolean colorMapTop3, boolean boldTop3, MarkupRenderer renderer) {
        StringBuilder report = new StringBuilder();
        report.append(renderer.large(renderer.bold("Results Per Player")));
        report.append(renderer.newline());

        final int srcCols = 2 + missions.size() + 2; // rank, name, map scores, total score, total time
        final int srcRows = highScores.size();
        final int dstCols = srcCols;
        final int dstRows = srcRows + 1; // for header
        int row = 0;
        int col;
        String[][] table = new String[dstRows][dstCols];
        final int COL_RANK = 0;
        final int COL_NAME = 1;
        final int COL_MAP1 = 2;
        final int COL_TIME = srcCols-1;
        final int COL_SCORE = srcCols-2;
        // add the column headers
        for (col = 0; col < dstCols; col++) {
            String header;

            if      (col == COL_RANK ) {header = "Rank";}
            else if (col == COL_NAME ) {header = "Name";}
            else if (col == COL_TIME ) {header = "Total Time";}
            else if (col == COL_SCORE) {header = "Total Score";}
            // Other columns are Map1-5 (converted to mission names)
            else {
                header = missions.get(col - COL_MAP1).code;
            }
            
            // Headers are in bold
            table[row][col] = renderer.bold(header);
        }
        row++;

        // Calculate sorting order for total time and total score
        Integer[] totalScores = new Integer[highScores.size()];
        Frames[]  totalTimes = new Frames[highScores.size()];
        int i = 0;
        for (HighScoreEntry highScoreEntry : highScores) {
            totalScores[i] = highScoreEntry.totalScore;
            totalTimes[i] = highScoreEntry.playedAllGames ? highScoreEntry.totalTime : null;
            i++;
        }
        int[] scoreRanks = SortUtil.getSortIndex(totalScores, SortSpec.HI_FIRST);
        int[] timeRanks = SortUtil.getSortIndex(totalTimes, SortSpec.LO_FIRST);

        // format some fancily (gold/silver/bronze rows for victors, bold for column max)
        i = 0;
        for (HighScoreEntry highScoreEntry : highScores) {
            int rank = i+1;
            col = 0;
            for (int dc = 0; dc < dstCols; dc++) {
                String cell;
                String color = "";
                boolean colorize = false;
                int rankInCol = i; // 0-based rank (topness) per column
                // The rank column has first, second, third place as gold, silver, bronze.
                if (dc == COL_RANK) {
                    cell = Integer.toString(rank);
                    colorize = true;
                    color = renderer.getVictorColor(i);
                } else if (dc == COL_NAME) {
                    // color the names of the top 3 people, not just their ranks
                    cell = highScoreEntry.user;
                    colorize = true;
                    color = renderer.getVictorColor(i);
                } else if (dc == COL_TIME) {
                    // TODO apply top-3 formatting for highest time
                    if (totalTimes[i] != null)
                        cell = totalTimes[i].toString();
                    else
                        cell = "";
                    rankInCol = timeRanks[i];
                } else if (dc == COL_SCORE) {
                    // TODO apply top-3 formatting for lowest time
                    cell = Integer.toString(totalScores[i]);
                    rankInCol = scoreRanks[i];
                } else {
                    // map score
                    final int mapnum = dc - COL_MAP1;
                    final Record record = scoresByGameByUser.get(highScoreEntry.user).get(mapnum);
                    if (record == null) {
                        cell = "";
                        rankInCol = -1;
                    } else {
                        cell = record.score.toString();
                        rankInCol = record.rank-1;
                    }
                }
                if (colorMapTop3) {
                    // in this scheme, the top 3 in each sorted column are colorized
                    // (they might also get bolded below if boldTop3 is set)
                    colorize = true;
                    color = renderer.getVictorColor(rankInCol);
                } else {
                    // In this scheme, the only special formatting for sorted columns
                    // is that the top cell gets bolded.
                    if (rankInCol == 0)
                        cell = renderer.bold(cell);
                }

                // got cell text, now maybe apply formatting.
                if (colorize && !color.isEmpty()) {
                    cell = renderer.color(cell, color);
                    if (boldTop3) cell = renderer.bold(cell); // If set, make all ranked cells bold.
                }
                // record cell in table
                table[row][col++] = cell;
            }
            row++;
            i++;
        }

        report.append(renderer.table(table, true));

        if (!colorMapTop3)
            report.append(renderer.italic("Best scores in Bold.")).append(renderer.newline());
        // no helper text needed for gold/silver/bronze

        report.append("Scores recorded at ").append(scoreTimestampStr).append(" forum time.");

        return report.toString();
    }

    public static String makeWikiResultsEntry(String tournamentTitle, String url, List<HighScoreEntry> highScores, MarkupRenderer renderer) {
        String ID = "NN"; //placeholder text
        String goldPlayer = "";
        String goldScore = "";
        if (highScores.size() > 0) {
            goldPlayer = highScores.get(0).user;
            goldScore = Integer.toString(highScores.get(0).totalScore);
        }
        String silverPlayer = "";
        String silverScore = "";
        if (highScores.size() > 1) {
            silverPlayer = highScores.get(1).user;
            silverScore = Integer.toString(highScores.get(1).totalScore);
        }
        String bronzePlayer = "";
        String bronzeScore = "";
        if (highScores.size() > 2) {
            bronzePlayer = highScores.get(2).user;
            bronzeScore = Integer.toString(highScores.get(2).totalScore);
        }
        final int cols = 9;
        String [][] tableRow = new String[1][cols];
        int row = 0, col = 0;
        tableRow[row][col++] = renderer.url(url, ID);
        tableRow[row][col++] = tournamentTitle;
        tableRow[row][col++] = goldPlayer;
        tableRow[row][col++] = silverPlayer;
        tableRow[row][col++] = bronzePlayer;
        tableRow[row][col++] = "";
        tableRow[row][col++] = renderer.tableRight(goldScore);
        tableRow[row][col++] = renderer.tableRight(silverScore);
        tableRow[row][col++] = renderer.tableRight(bronzeScore);

        return renderer.table(tableRow, false); // no first header since this is just one row in the table
    }

}
