package codemaptournament;

import java.net.URL;
import java.util.Date;

/**
 * Information about a week's 5-map tournament (title, url, close date, etc).
 * Sorts by closing date, most recent first.
 * @author Paul
 */
public class WeekInfo implements Comparable<WeekInfo> {

    /** raw text of the thread's title */
    String fulltext;
    /** subject of the tournament */
    String topic;
    /**When the tournament ends.
     * If now is before cutoff, the tournament is still open.
     * Usually this is 0000 on Monday (right after 2359 on Sunday)
     */
    Date cutoff;
    /** human-readable closing date (what was given in the forum post) */
    String closing;
    /** link to the thread's url */
    URL link;

    public WeekInfo(String fulltext, String topic, Date cutoff, String closing, URL link) {
        this.fulltext = fulltext;
        this.topic = topic;
        this.cutoff = cutoff;
        this.closing = closing;
        this.link = link;
    }

    /** sort weeks from recent to old */
    public int compareTo(WeekInfo o) {
        return -1 * cutoff.compareTo(o.cutoff);
        // this it the same as sorting by closing date, but I store the closing date as text
        // and the cutoff date (+1day) as a Date
    }

    /** compare cutoff time to current time */
    public boolean isOpen() {
        Date now = new Date();
        return now.before(cutoff);
    }


    /** label with format: "(Topic) closing (Date string)   (maybe [Open])" */
    @Override
    public String toString() {
        StringBuilder label = new StringBuilder();
        label.append(topic);
        label.append(" closing ");
        label.append(closing);
        if (isOpen())
            label.append("  [Open]");
        return label.toString();
    }
}
