package codemaptournament;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *
 * @author Paul
 */
public class WikiTable {
    public static final URL wikipageURL;
    static {
        try {
            wikipageURL = new URL("http://knucklecracker.com/wiki/doku.php?do=export_raw&id=cw2:weekly_5_map_tournament");

            //"http://knucklecracker.com/wiki/doku.php?do=export_raw&id=cw2:weekly_5_map_tournament"?
        } catch (MalformedURLException ex) {
            throw new RuntimeException(ex);
        }
    }
    // Find the header, then capture everything until either EOF or a line that doesn't start with ^ or |
    private static final String sTournamentHeader = "====\\s+Past Tournament Results\\s+====\\s+";
    private static final String sFirstLine = "^([^\\n]*\\n)";
    private static final String sRestLine = "^(.*?)(?:\\Z|^[^\\^|])";
    Pattern pPastTournaments = Pattern.compile(sTournamentHeader + sFirstLine + sRestLine, Pattern.DOTALL | Pattern.CASE_INSENSITIVE | Pattern.MULTILINE);
    private static final String sCapInt = "\\s*(\\d*)\\s*\\|";
    private static final String sCapStr = "\\s*(.*?)\\s*\\|";
    private static final String sSkipStr = ".*?\\|";
    //                                            number     theme     Gold      Silver    Bronze    (skip)    GoldScore  Silver   Bronze
    Pattern pParseTable = Pattern.compile("\\|" + sCapInt + sCapStr + sCapStr + sCapStr + sCapStr + sSkipStr + sCapInt + sCapInt + sCapInt);
    
    public List<MedalCountEntry> parseWikiPage() throws Exception {
        String pagedata = NetUtil.readCompletely(wikipageURL, NetUtil.KC_WIKI_ENCODING, NetUtil.FAKE_USERAGENT);
        Matcher mPastTournaments = pPastTournaments.matcher(pagedata);
        if (mPastTournaments.find()) {
//            String wikiHeader = mPastTournaments.group(1);
            String wikitext = mPastTournaments.group(2);
//            System.out.println(wikitext);
            Matcher mParseTable = pParseTable.matcher(wikitext);
            List<WikiTournamentRecord> records = new ArrayList<WikiTournamentRecord>();
            WikiTournamentRankings ranker = new WikiTournamentRankings();
            while (mParseTable.find()) {
                WikiTournamentRecord record = new WikiTournamentRecord(mParseTable);
                ranker.accumulate(record);
            }
            List<MedalCountEntry> tournamentRankings = ranker.getTournamentRankings();
            return tournamentRankings;
        } else {
            return Collections.EMPTY_LIST;
        }
    }

    private static class StringLengthMeasurer {
        int longestLength = 0;
        void accumulate(String s) {
            final int l = s.length();
            if (l > longestLength) longestLength = l;
        }
        void accumulate(int number) {
            accumulate(Integer.toString(number));
        }
    }

    public static String formatForWiki(List<MedalCountEntry> tournamentRankings) {
        StringBuilder sb = new StringBuilder();

        // Determine string lengths of each column for proper alignment
        // (not necessary but makes it look pretty)
        StringLengthMeasurer nameMeasurer = new StringLengthMeasurer();
        StringLengthMeasurer goldMeasurer = new StringLengthMeasurer();
        StringLengthMeasurer silverMeasurer = new StringLengthMeasurer();
        StringLengthMeasurer bronzeMeasurer = new StringLengthMeasurer();
        StringLengthMeasurer pointsMeasurer = new StringLengthMeasurer();

        for (MedalCountEntry medalCountEntry : tournamentRankings) {
            nameMeasurer.accumulate(medalCountEntry.playerName);
            goldMeasurer.accumulate(medalCountEntry.goldCount);
            silverMeasurer.accumulate(medalCountEntry.silverCount);
            bronzeMeasurer.accumulate(medalCountEntry.bronzeCount);
            pointsMeasurer.accumulate(medalCountEntry.calcScore());
        }
        String format = String.format("|%%-%ds|%%-%dd|%%-%dd|%%-%dd|%%-%dd|",
                nameMeasurer.longestLength + 1,
                goldMeasurer.longestLength + 1,
                silverMeasurer.longestLength + 1,
                bronzeMeasurer.longestLength + 1,
                pointsMeasurer.longestLength + 1);
        for (MedalCountEntry medalCountEntry : tournamentRankings) {
            sb.append(String.format(format,
                    medalCountEntry.playerName,
                    medalCountEntry.goldCount,
                    medalCountEntry.silverCount,
                    medalCountEntry.bronzeCount,
                    medalCountEntry.calcScore()));
//            sb.append(medalCountEntry.formatForWiki());
            sb.append("\n");
        }
        return sb.toString();
    }
    
    public static void main(String[] args) {
        try {
            WikiTable w = new WikiTable();
            List<MedalCountEntry> tournamentRankings = w.parseWikiPage();
            String wikiRankingTable = WikiTable.formatForWiki(tournamentRankings);
            System.out.println(wikiRankingTable);
        }
        catch (Exception ex) {
            Logger.getLogger(WikiTable.class.getName()).log(Level.SEVERE, null, ex);
        }
        
    }

}
